@@ -33,6 +33,7 @@ from .detection import (
 )
 from .enriched_event import PantherEvent
 from .exceptions import FunctionReturnTypeError, UnknownDestinationError
+from .filter import Filter
 
 # Temporary alias for compatibility
 get_logger = logging.getLogger
@@ -144,13 +145,13 @@ class Detection(ABC):
             self.detection_dedup_period_mins = config["dedupPeriodMinutes"]
 
         if not ("tags" in config) or not isinstance(config["tags"], list):
-            self.detection_tags: List[str] = list()
+            self.detection_tags: List[str] = []
         else:
             config["tags"].sort()
             self.detection_tags = config["tags"]
 
         if "reports" not in config or not config.get("reports"):
-            self.detection_reports: Dict[str, List[str]] = dict()
+            self.detection_reports: Dict[str, List[str]] = {}
         else:
             # Reports are Dict[str, List[str]]
             # Sorting the List before setting it
@@ -174,6 +175,21 @@ class Detection(ABC):
         self._default_dedup_string = "defaultDedupString:{}".format(self.detection_id)
         self._auxiliary_function_definitions = self._check_defined_functions()
 
+        if config.get("filters", None) is None:
+            self.filters: List[Filter] = []
+        elif isinstance(config["filters"], Filter):
+            self.filters = [config["filters"]]
+        elif isinstance(config["filters"], list) and all(
+            isinstance(f, Filter) for f in config["filters"]
+        ):
+            self.filters = config["filters"]
+        else:
+            self._setup_exception = TypeError(
+                "expected 'filters' to be of type Union[List[Filter], Filter, None]"
+            )
+
+        self.skip_filters: bool = bool(config.get("skip_filters"))
+
     @staticmethod
     def _load_detection(identifier: str, config: Mapping) -> ModuleType:
         """Load the detection code as Python module.
@@ -259,12 +275,24 @@ class Detection(ABC):
             return detection_result
 
         try:
-            detection_result.detection_output = self.matcher_function(event)
+            # Run filters before running matcher function
+            if self._filters_should_run():
+                # all filters must return True in order to run the match function (match func is basically a filter)
+                detection_result.filter_output = all(f.filter(event) for f in self.filters)
+        except Exception as err:  # pylint: disable=broad-except
+            detection_result.filter_exception = err
+
+        try:
+            if detection_result.filters_did_not_match:
+                detection_result.detection_output = False
+            elif detection_result.filters_not_ran or detection_result.all_filters_matched:
+                detection_result.detection_output = self.matcher_function(event)
         except Exception as err:  # pylint: disable=broad-except
             detection_result.detection_exception = err
 
         detection_result.trigger_alert = (
             detection_result.detection_output is self.matcher_alert_value
+            and (detection_result.all_filters_matched or detection_result.filters_not_ran)
         )
 
         if batch_mode and not detection_result.trigger_alert:
@@ -372,6 +400,14 @@ class Detection(ABC):
 
         return detection_result
 
+    def _filters_should_run(self) -> bool:
+        return (
+            # Only running on basic rules for now
+            self.detection_type == TYPE_RULE
+            and len(self.filters) > 0
+            and not self.skip_filters
+        )
+
     def _get_alert_context(
         self, event: Mapping, use_default_on_exception: bool = True
     ) -> Optional[str]:
@@ -486,7 +522,7 @@ class Detection(ABC):
     ) -> Optional[List[str]]:
         try:
             command = getattr(self._module, DESTINATIONS_FUNCTION)
-            destinations = self._run_command(command, event, list())
+            destinations = self._run_command(command, event, [])
         except Exception as err:  # pylint: disable=broad-except
             if use_default_on_exception:
                 self.logger.info("destinations method raised exception. Exception: %s", err)
@@ -710,5 +746,6 @@ class Rule(Detection):
             self._module, self.matcher_function_name
         ):
             return True
+
         command = getattr(self._module, self.matcher_function_name)
         return self._run_command(command, event, bool)
